#ifndef INCLUDE_GIM_PUBLIC_H
#define INCLUDE_GIM_PUBLIC_H
#ifdef __cplusplus
extern "C" {
#endif
/* START PUBLIC */

#include <sys/types.h>

void
gim_init (void);

struct gim_client_masks {
  unsigned minmod;
  unsigned maxmod;
  unsigned handle;
  unsigned pass;
};

int
gim_client_connect (const struct gim_client_masks* p_masks, int vt, const char* device);

#define GIM_CONNECT_CMD_FAILURE GIM_CMD_FAILURE
#define GIM_CONNECT_ERR_GREETING -2
#define GIM_CONNECT_ERR_VERSION -3
#define GIM_CONNECT_NO_CTTY -4
#define GIM_CONNECT_ERR_UNIX GIM_ERR_UNIX
#define GIM_CONNECT_ERR_UNKNOWN GIM_ERR_UNKNOWN

                                /* returns 0 on success */
int
gim_client_close (int fd);

#define GIM_CLOSE_SUCCESS GIM_SUCCESS
#define GIM_CLOSE_CMD_FAILURE GIM_CMD_FAILURE
#define GIM_CLOSE_ERR_UNKNOWN GIM_ERR_UNKNOWN

int
gim_client_set_masks (int fd, const struct gim_client_masks* p_masks);

#define GIM_SET_MASKS_SUCCESS GIM_SUCCESS
#define GIM_SET_MASKS_CMD_FAILURE GIM_CMD_FAILURE
#define GIM_SET_MASKS_ERR_UNKNOWN GIM_ERR_UNKNOWN

int
gim_client_get_masks (int fd, struct gim_client_masks* p_masks);

#define GIM_GET_MASKS_SUCCESS GIM_SUCCESS
#define GIM_GET_MASKS_CMD_FAILURE GIM_CMD_FAILURE
#define GIM_GET_MASKS_ERR_UNKNOWN GIM_ERR_UNKNOWN

int
gim_client_draw_pointer (int fd, int x, int y);

#define GIM_DRAW_POINTER_SUCCESS GIM_SUCCESS
#define GIM_DRAW_POINTER_CMD_FAILURE GIM_CMD_FAILURE
#define GIM_DRAW_POINTER_ERR_UNKNOWN GIM_ERR_UNKNOWN

int
gim_client_stop (int fd);

#define GIM_STOP_SUCCESS GIM_SUCCESS
#define GIM_STOP_CMD_FAILURE GIM_CMD_FAILURE
#define GIM_STOP_ERR_UNKNOWN GIM_ERR_UNKNOWN

extern int
gim_client_start (int fd);

#define GIM_START_SUCCESS GIM_SUCCESS
#define GIM_START_CMD_FAILURE GIM_CMD_FAILURE
#define GIM_START_ERR_UNKNOWN GIM_ERR_UNKNOWN

/* returns length of copied string (even if it is not actually copied
   because it's too long for the supplied buffer */

int
gim_client_devinfo (int fd, char* info, size_t info_size);

#define GIM_DEVINFO_CMD_FAILURE GIM_CMD_FAILURE
#define GIM_DEVINFO_ERR_UNKNOWN GIM_ERR_UNKNOWN

int
gim_client_servinfo (int fd, char* info, size_t info_size);

#define GIM_SERVINFO_CMD_FAILURE GIM_CMD_FAILURE
#define GIM_SERVINFO_ERR_UNKNOWN GIM_ERR_UNKNOWN

int
gim_client_disable_paste (int fd);

#define GIM_DISABLE_PASTE_SUCCESS GIM_SUCCESS
#define GIM_DISABLE_PASTE_CMD_FAILURE GIM_CMD_FAILURE
#define GIM_DISABLE_PASTE_ERR_UNKNOWN GIM_ERR_UNKNOWN

/* END PUBLIC */
/* START PUBLIC */

#include <sys/time.h>
#include <stdio.h>

enum {
  gim_move = 1,
  gim_drag,
  gim_down,
  gim_up,
  gim_dragged_click,
  gim_single,
  gim_double,
  gim_triple,
  gim_enter,
  gim_leave,
  gim_last_ev
};

#define EVDEF(x) x ## _ev = (1 << ((x) - 1))

enum  gim_event_kind_t {
  EVDEF(gim_move),
  EVDEF(gim_drag),
  gim_position_ev = (gim_move_ev | gim_drag_ev),
  EVDEF(gim_down),
  EVDEF(gim_up),
  gim_button_ev = (gim_down_ev | gim_up_ev),
  gim_raw_ev = (gim_position_ev | gim_button_ev),
  EVDEF(gim_dragged_click),
  EVDEF(gim_single),
  EVDEF(gim_double),
  EVDEF(gim_triple),
  gim_multi_ev = (gim_single_ev | gim_double_ev | gim_triple_ev),
  EVDEF(gim_enter),
  EVDEF(gim_leave),
  gim_synth_ev = (gim_enter_ev | gim_leave_ev),
  EVDEF(gim_last_ev)
};

enum {
  gim_top = 1,
  gim_left,
  gim_bottom,
  gim_right,
  gim_last_mg
};

#define MGDEF(x) x ## _mg = (1 << ((x) - 1))  

enum gim_margin_t {
  MGDEF(gim_top),
  MGDEF(gim_left),
  gim_topleft_mg = (gim_top_mg | gim_left_mg),
  MGDEF(gim_bottom),
  gim_bottomleft_mg = (gim_bottom_mg | gim_left_mg),
  MGDEF(gim_right),
  gim_topright_mg = (gim_top_mg | gim_right_mg),
  gim_bottomright_mg = (gim_bottom_mg | gim_right_mg),
  gim_vertical_mg = (gim_top_mg | gim_bottom_mg),
  gim_horizontal_mg = (gim_left_mg | gim_right_mg),
  gim_none_mg = 0,
  MGDEF(gim_last_mg)
};

struct gim_event_t {
  enum gim_event_kind_t kind;
  int x;                        /* absolute coordinates */
  int y;
  int z;
  unsigned state;               /* mask of first 32 pressed buttons */
  unsigned modifiers;
  struct timeval time;          /* timestamp */
  int vt;
  union {
    struct {
      int x;
      int y;
      int z;
      enum gim_margin_t margin;
    } d;                        /* only for moves and drags */
    struct {
      int which;
      int repeat;
    } b;                        /* only for downs and ups */
  } u;
};

int
gim_get_event (int fd, struct gim_event_t* p_ev);

#define GIM_GET_EVENT_SUCCESS GIM_SUCCESS
#define GIM_GET_EVENT_ERR_PROTOCOL -300
#define GIM_GET_EVENT_ERR_UNKNOWN GIM_ERR_UNKNOWN

struct gim_handler_data_t {
  int morekeys;
  int is_fake;
  void* extra_data;
};

typedef int (*gim_handler)(struct gim_event_t*, struct gim_handler_data_t*);

/* this should really not be public, but it is needed for the wgetch macro below */
typedef int (*gim_getcfun_t)(void* p);

int
gim_event_select (int fd, gim_handler h, struct gim_handler_data_t* hdata,
                  gim_getcfun_t p, void* arg);

int 
gim_getchar (int fd, gim_handler h, struct gim_handler_data_t* hdata);

/* in this way we avoid the dependency on curses, hee hee */
#define gim_wgetch(fd, h, hdata, w) (gim_event_select((fd), (h), (hdata), (int (*)(void*))wgetch, (w)))

struct gim_region_t {
  int x_min, x_max, y_min, y_max;
  unsigned min_mod, max_mod;
  enum gim_event_kind_t handle;
  gim_handler handler_fun;
  void* extra_data;
  int active;
  struct gim_region_t* prev;
  struct gim_region_t* next;
};

void
gim_init_region (struct gim_region_t* p_reg);

struct gim_region_t*
gim_alloc_region (void);

void
gim_put_region (struct gim_region_t* p_reg, struct gim_region_t* after);

int
gim_region_handler (struct gim_event_t* p_ev, struct gim_handler_data_t* hdata);

/* END PUBLIC */
/* START PUBLIC */

#define GIM_SUCCESS 0
#define GIM_CMD_FAILURE -1
#define GIM_ERR_UNKNOWN -100
#define GIM_ERR_UNIX -200

const char*
gim_error_string (int status);

/* END PUBLIC */
#ifdef __cplusplus
}
#endif
#endif
